<?php

namespace Drupal\cms_content_sync\Controller;

use Drupal\Core\Url;
use Drupal\Core\Controller\ControllerBase;
use Drupal\cms_content_sync\SyncCoreFlowExport;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Push changes controller.
 */
class FlowExport extends ControllerBase {

  /**
   * Export flow.
   */
  public function export($cms_content_sync_flow) {
    /**
     * @var \Drupal\cms_content_sync\Entity\Flow $flow
     */
    $flow = \Drupal::entityTypeManager()
      ->getStorage('cms_content_sync_flow')
      ->load($cms_content_sync_flow);

    if (empty($flow->getUsedPools())) {
      \Drupal::messenger()->addError("This Flow doesn't use any Pools so nothing will be pushed or pulled. Please assign a Pool to this Flow first.");

      return RedirectResponse::create(
        Url::fromRoute('entity.cms_content_sync_flow.collection')->toString()
      );
    }

    $exporter = new SyncCoreFlowExport($flow);

    $batch      = $exporter->prepareBatch();
    $operations = [];
    for ($i = 0; $i < $batch->count(); $i++) {
      $operations[] = [
        [$batch->get($i), 'execute'],
        [],
      ];
    }

    $batch = [
      'title' => t('Export configuration'),
      'operations' => $operations,
      'finished' => '\Drupal\cms_content_sync\Controller\FlowExport::batchFinished',
    ];
    batch_set($batch);

    return batch_process(Url::fromRoute('entity.cms_content_sync_flow.collection'));
  }

  /**
   * Batch export finished callback.
   *
   * @param $success
   * @param $results
   * @param $operations
   */
  public static function batchFinished($success, $results, $operations) {
    if ($success) {
      $message = t('Flow has been exported.');
    }
    else {
      $message = t('Flow export failed.');
    }

    \Drupal::messenger()->addMessage($message);
  }

}
